﻿using System;
using System.ComponentModel;
using Xamarin.Forms;
using Plugin.CSJPOSLib;
using Xamarin.Essentials;

namespace Sample1
{
    // Learn more about making custom code visible in the Xamarin.Forms previewer
    // by visiting https://aka.ms/xamarinforms-previewer
    [DesignTimeVisible(true)]
    public partial class MainPage : ContentPage
    {
        private IScanner _scanner;
        public IScanner Scanner
        {
            get
            {
                if (_scanner == null)
                {
                    _scanner = CrossCSJPOSLib.CreateScanner();
                }

                return _scanner;
            }
        }



        public MainPage()
        {
            InitializeComponent();

            this.InterfacePicker.SelectedIndex = 0;
            this.DisplayInterfacePicker.SelectedIndex = 0;
            this.ScannerInterfacePicker.SelectedIndex = 0;

            this.AddrEdit.Text = "";
            this.DisplayAddrEdit.Text = "";
            this.ScannerAddrEdit.Text = "";
            this.portEdit.Text = "9100";
            this.DisplayPortEdit.Text = "9200";
            this.ScannerPortEdit.Text = "9210";

            this.StatusText.Text = "Stopped";
            this.StatusFrame.BackgroundColor = Color.LightGray;
        }

        private void MessageBox(string message, string title)
        {
             var result = DisplayAlert(title, message, "OK");
        }

        private void OnPrinterSelectedChanged(object sender, EventArgs args)
        {
        }
        private void OnDisplaySelectedChanged(object sender, EventArgs args)
        {
            var obj = (Picker)sender;
            if (3 == obj.SelectedIndex)
            {
                this.DisplayPortEdit.Text = "";
            }
        }
        private void OnScannerSelectedChanged(object sender, EventArgs args)
        {
            var obj = (Picker)sender;
            if (3 == obj.SelectedIndex)
            {
                this.ScannerPortEdit.Text = "";
            }
        }

        private void OnSamplePrint1(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.AddrEdit.Text;
            port = this.portEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                SamplePrint1(this.InterfacePicker.SelectedIndex, address, portnum);
            }
            else
            {
                SamplePrint1(this.InterfacePicker.SelectedIndex, address);
            }
        }

        private void OnSamplePrint2(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.AddrEdit.Text;
            port = this.portEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                SamplePrint2(this.InterfacePicker.SelectedIndex, address, portnum);
            }
            else
            {
                SamplePrint2(this.InterfacePicker.SelectedIndex, address);
            }
        }

        private void OnSampleDisplay1(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.DisplayAddrEdit.Text;
            port = this.DisplayPortEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                SampleDisplay1(this.DisplayInterfacePicker.SelectedIndex, address, portnum);
            }
            else
            {
                SampleDisplay1(this.DisplayInterfacePicker.SelectedIndex, address);
            }
        }

        private void OnSampleDisplay2(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.DisplayAddrEdit.Text;
            port = this.DisplayPortEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                SampleDisplay2(this.DisplayInterfacePicker.SelectedIndex, address, portnum);
            }
            else
            {
                SampleDisplay2(this.DisplayInterfacePicker.SelectedIndex, address);
            }
        }

        private void OnScannerConnect(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.ScannerAddrEdit.Text;
            port = this.ScannerPortEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                ScannerConnect(this.ScannerInterfacePicker.SelectedIndex, address, portnum);
            }
            else
            {
                ScannerConnect(this.ScannerInterfacePicker.SelectedIndex, address);
            }
        }

        private void OnScannerDisconnect(object sender, EventArgs args)
        {
            ScannerDisconnect();
        }



        int SamplePrint1(int connectType, string address, int port)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            IESCPOSPrinter printer = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateESCPOSPrinter();

            result = printer.Connect(connectType, address, port);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // character set
                _ = printer.SetEncoding("IBM437");       // OEM-US    Enable this line if your printer's MSW9-1 is "CP437".
            //  _ = printer.SetEncoding("UTF-8");        // Unicode   Enable this line if your printer supports "UTF-8" and prints characters in "UTF-8" code.
            //  _ = printer.SetEncoding("Shift_JIS");    // Japanese  日本語を "Shift_JIS" コードで印字する場合は、この行を有効にしてください.

                // start transaction (batch)
                _ = printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

                // printText
                var appName = AppInfo.Name;
                _ = printer.PrintText(appName + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                _ = printer.PrintText("- Sample Print 1 -\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);
                _ = printer.PrintText("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                //printQRCode
                _ = printer.PrintQRCode("http://www.citizen-systems.co.jp", 6, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_RIGHT);

                // partical cut with pre-feed
                _ = printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // end transaction (batch)
                result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

                // discoonect
                _ = printer.Disconnect();


                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    MessageBox("Transaction Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Printer Error : " + result, "Error");
            }

            return result;
        }

        int SamplePrint1(int connectType, string address)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            IESCPOSPrinter printer = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateESCPOSPrinter();

            result = printer.Connect(connectType, address);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // character set
                _ = printer.SetEncoding("IBM437");       // OEM-US    Enable this line if your printer's MSW9-1 is "CP437".
                                                         //  _ = printer.SetEncoding("UTF-8");        // Unicode   Enable this line if your printer supports "UTF-8" and prints characters in "UTF-8" code.
                                                         //  _ = printer.SetEncoding("Shift_JIS");    // Japanese  日本語を "Shift_JIS" コードで印字する場合は、この行を有効にしてください.

                // start transaction (batch)
                _ = printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

                // printText
                var appName = AppInfo.Name;
                _ = printer.PrintText(appName + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                _ = printer.PrintText("- Sample Print 1 -\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);
                _ = printer.PrintText("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                //printQRCode
                _ = printer.PrintQRCode("http://www.citizen-systems.co.jp", 6, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_RIGHT);

                // partical cut with pre-feed
                _ = printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // end transaction (batch)
                result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

                // discoonect
                _ = printer.Disconnect();


                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    MessageBox("Transaction Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Printer Error : " + result, "Error");
            }

            return result;
        }

        int SamplePrint2(int connectType, string address, int port)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            IESCPOSPrinter printer = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateESCPOSPrinter();

            result = printer.Connect(connectType, address, port);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // character set
                _ = printer.SetEncoding("IBM437");       // OEM-US    Enable this line if your printer's MSW9-1 is "CP437".
            //  _ = printer.SetEncoding("UTF-8");        // Unicode   Enable this line if your printer supports "UTF-8" and prints characters in "UTF-8" code.
            //  _ = printer.SetEncoding("Shift_JIS");    // Japanese  日本語を "Shift_JIS" コードで印字する場合は、この行を有効にしてください.

                // start transaction (batch)
                _ = printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

                var appName = AppInfo.Name;

                // printText
                _ = printer.PrintText(appName + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                string charCode = "1b";
                int charCode16 = Convert.ToInt32(charCode, 16);
                char c = Convert.ToChar(charCode16);
                string sample2 = string.Format("{0:D1}|2vC{1:D1}|cA- Sample Print 2 -\n", c, c);
                _ = printer.PrintNormal(sample2);
                _ = printer.PrintText("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                // start page mode
                _ = printer.PageModePrint(ESCPOSConst.CMP_PM_PAGE_MODE);

                // set offset value in pagemode
                _ = printer.SetPageModeVerticalPosition(0);
                _ = printer.SetPageModeHorizontalPosition(0);

                // set print direction in pagemode
                _ = printer.SetPageModePrintDirection(ESCPOSConst.CMP_PD_TOP_TO_BOTTOM);

                // set printable area in pagemode
                _ = printer.SetPageModePrintArea("500,0,76,800");


                string receipt = string.Format("{0:D1}|4C- Receipt -\n", c);
                _ = printer.PrintNormal(receipt);
                _ = printer.SetPageModePrintArea("260,0,120,800");
                _ = printer.PrintText("$ 299.99- \n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD, ESCPOSConst.CMP_TXT_4WIDTH | ESCPOSConst.CMP_TXT_4HEIGHT);
                _ = printer.SetPageModePrintArea("88,0,88,560");
                _ = printer.PrintText(" CITIZEN SYSTEMS\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_3HEIGHT);
                _ = printer.SetPageModePrintArea("0,0,98,480");
                _ = printer.PrintBarCode("123456789012", ESCPOSConst.CMP_BCS_UPCA, 64, 4, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
                _ = printer.SetPageModePrintArea("0,600,192,192");
                _ = printer.PrintQRCode("http://www.citizen-systems.co.jp/", 5, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_LEFT);

                // end page mode
                _ = printer.PageModePrint(ESCPOSConst.CMP_PM_NORMAL);


                // partical cut with pre-feed
                _ = printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // end transaction (batch)
                result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

                // discoonect
                _ = printer.Disconnect();


                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    MessageBox("Transaction Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Printer Error : " + result, "Error");
            }

            return result;
        }

        int SamplePrint2(int connectType, string address)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            IESCPOSPrinter printer = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateESCPOSPrinter();

            result = printer.Connect(connectType, address);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // character set
                _ = printer.SetEncoding("IBM437");       // OEM-US    Enable this line if your printer's MSW9-1 is "CP437".
                                                         //  _ = printer.SetEncoding("UTF-8");        // Unicode   Enable this line if your printer supports "UTF-8" and prints characters in "UTF-8" code.
                                                         //  _ = printer.SetEncoding("Shift_JIS");    // Japanese  日本語を "Shift_JIS" コードで印字する場合は、この行を有効にしてください.

                // start transaction (batch)
                _ = printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

                var appName = AppInfo.Name;

                // printText
                _ = printer.PrintText(appName + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                string charCode = "1b";
                int charCode16 = Convert.ToInt32(charCode, 16);
                char c = Convert.ToChar(charCode16);
                string sample2 = string.Format("{0:D1}|2vC{1:D1}|cA- Sample Print 2 -\n", c, c);
                _ = printer.PrintNormal(sample2);
                _ = printer.PrintText("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                // start page mode
                _ = printer.PageModePrint(ESCPOSConst.CMP_PM_PAGE_MODE);

                // set offset value in pagemode
                _ = printer.SetPageModeVerticalPosition(0);
                _ = printer.SetPageModeHorizontalPosition(0);

                // set print direction in pagemode
                _ = printer.SetPageModePrintDirection(ESCPOSConst.CMP_PD_TOP_TO_BOTTOM);

                // set printable area in pagemode
                _ = printer.SetPageModePrintArea("500,0,76,800");


                string receipt = string.Format("{0:D1}|4C- Receipt -\n", c);
                _ = printer.PrintNormal(receipt);
                _ = printer.SetPageModePrintArea("260,0,120,800");
                _ = printer.PrintText("$ 299.99- \n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD, ESCPOSConst.CMP_TXT_4WIDTH | ESCPOSConst.CMP_TXT_4HEIGHT);
                _ = printer.SetPageModePrintArea("88,0,88,560");
                _ = printer.PrintText(" CITIZEN SYSTEMS\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_3HEIGHT);
                _ = printer.SetPageModePrintArea("0,0,98,480");
                _ = printer.PrintBarCode("123456789012", ESCPOSConst.CMP_BCS_UPCA, 64, 4, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
                _ = printer.SetPageModePrintArea("0,600,192,192");
                _ = printer.PrintQRCode("http://www.citizen-systems.co.jp/", 5, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_LEFT);

                // end page mode
                _ = printer.PageModePrint(ESCPOSConst.CMP_PM_NORMAL);


                // partical cut with pre-feed
                _ = printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // end transaction (batch)
                result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

                // discoonect
                _ = printer.Disconnect();


                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    MessageBox("Transaction Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Printer Error : " + result, "Error");
            }

            return result;
        }

        int SampleDisplay1(int connectType, string address, int port)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            ILineDisplay display = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateLineDisplay();

            result = display.Connect(connectType, address, port); 

            if (LineDisplayConst.CDP_SUCCESS == result)
            {
            //  _ = display.SetEncoding("US-ASCII");      // US-ASCII  Enable this line if the display data is ASCII code only. 
            //  _ = display.SetEncoding("Shift_JIS");     // Japanese  日本語を "Shift_JIS" コードで表示する場合は、この行を有効にしてください.

                _ = display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
                _ = display.DisplayText("New York steak", true);
                _ = display.MoveCursor(-1, 1);
                result = display.DisplayText("$89.99-");

                // discoonect
                _ = display.Disconnect();


                if (result != LineDisplayConst.CDP_SUCCESS)
                {
                    MessageBox("displayText Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Display Error : " + result, "Error");
            }

            return result;
        }
        int SampleDisplay1(int connectType, string address)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            ILineDisplay display = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateLineDisplay();

            result = display.Connect(connectType, address);

            if (LineDisplayConst.CDP_SUCCESS == result)
            {
                //  _ = display.SetEncoding("US-ASCII");      // US-ASCII  Enable this line if the display data is ASCII code only. 
                //  _ = display.SetEncoding("Shift_JIS");     // Japanese  日本語を "Shift_JIS" コードで表示する場合は、この行を有効にしてください.

                _ = display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
                _ = display.DisplayText("New York steak", true);
                _ = display.MoveCursor(-1, 1);
                result = display.DisplayText("$89.99-");

                // discoonect
                _ = display.Disconnect();


                if (result != LineDisplayConst.CDP_SUCCESS)
                {
                    MessageBox("displayText Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Display Error : " + result, "Error");
            }

            return result;
        }

        int SampleDisplay2(int connectType, string address, int port)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            ILineDisplay display = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateLineDisplay();

            result = display.Connect(connectType, address, port);

            if (LineDisplayConst.CDP_SUCCESS == result)
            {

                _ = display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
                _ = display.DisplayText("Thank you so much!");
                _ = display.SetCursorPosition(13, 2);
                result = display.DisplayText("$299.99-", true);

                // discoonect
                _ = display.Disconnect();


                if (result != LineDisplayConst.CDP_SUCCESS)
                {
                    MessageBox("displayText Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Display Error : " + result, "Error");
            }

            return result;
        }
        int SampleDisplay2(int connectType, string address)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            ILineDisplay display = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateLineDisplay();

            result = display.Connect(connectType, address);

            if (LineDisplayConst.CDP_SUCCESS == result)
            {

                _ = display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
                _ = display.DisplayText("Thank you so much!");
                _ = display.SetCursorPosition(13, 2);
                result = display.DisplayText("$299.99-", true);

                // discoonect
                _ = display.Disconnect();


                if (result != LineDisplayConst.CDP_SUCCESS)
                {
                    MessageBox("displayText Error : " + result, "Error");
                }
            }
            else
            {
                MessageBox("Connect or Display Error : " + result, "Error");
            }

            return result;
        }

        int ScannerConnect(int connectType, string address, int port)
        {
            int result = ScannerConst.CSC_SUCCESS;

            _ = this.Scanner.SetDataEventCallback(DataEventHandler);
            _ = this.Scanner.SetStatusUpdateEventCallback(StatusUpdateEventHandler);

            result = this.Scanner.Connect(connectType, address, port);

            if (ScannerConst.CSC_SUCCESS != result)
            {
                MessageBox("Connect or Scanner Error : " + result, "Error");
            }

            return result;
        }
        int ScannerConnect(int connectType, string address)
        {
            int result = ScannerConst.CSC_SUCCESS;

            _ = this.Scanner.SetDataEventCallback(DataEventHandler);
            _ = this.Scanner.SetStatusUpdateEventCallback(StatusUpdateEventHandler);

            result = this.Scanner.Connect(connectType, address);

            if (ScannerConst.CSC_SUCCESS != result)
            {
                MessageBox("Connect or Scanner Error : " + result, "Error");
            }

            return result;
        }

        int ScannerDisconnect()
        {
            int result = ScannerConst.CSC_SUCCESS;

            _ = this.Scanner.SetStatusUpdateEventCallback(null);
            _ = this.Scanner.SetDataEventCallback(null);

            _ = this.Scanner.Disconnect();

            this.StatusText.Text = "Stopped";
            this.StatusFrame.BackgroundColor = Color.LightGray;

            return result;
        }

        public void DataEventHandler(byte[] data)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                System.Diagnostics.Debug.WriteLine("OnDataEvent");

                string tmp = System.Text.Encoding.GetEncoding("us-ascii").GetString(data);      // US-ASCII  Enable this line if the scan data is ASCII code only.
            //  string tmp = System.Text.Encoding.GetEncoding("shift_jis").GetString(data);     // Japanese  スキャン データに "Shift_JIS" コードの日本語が含まれる場合は、この行を有効にしてください.

                MessageBox(tmp, "Scan Call back:");
            });
        }

        public void StatusUpdateEventHandler(int status)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                if (ScannerConst.CSC_SUE_POWER_ONLINE == status)
                {
                    this.StatusText.Text = "Online";
                    this.StatusFrame.BackgroundColor = Color.FromRgb(0, 255, 0);
                }
                else
                {
                    this.StatusText.Text = "Off-line";
                    this.StatusFrame.BackgroundColor = Color.Red;
                }
            });
        }
    }
}


