using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Forms;
using Plugin.CSJPOSLib;
using Xamarin.Essentials;
using System.Reflection;
using System.IO;

namespace Sample2
{
    // Learn more about making custom code visible in the Xamarin.Forms previewer
    // by visiting https://aka.ms/xamarinforms-previewer
    [DesignTimeVisible(true)]
    public partial class MainPage : ContentPage
    {
        IPlatformInterface platform = Xamarin.Forms.DependencyService.Get<IPlatformInterface>();

        private IESCPOSPrinter _printer;
        public IESCPOSPrinter Printer
        {
            get
            {
                if (_printer == null)
                {
                    _printer = CrossCSJPOSLib.CreateESCPOSPrinter();
                }

                return _printer;
            }
        }

        private ILineDisplay _display;
        public ILineDisplay Display
        {
            get
            {
                if (_display == null)
                {
                    _display = CrossCSJPOSLib.CreateLineDisplay();
                }

                return _display;
            }
        }

        private IScanner _scanner;
        public IScanner Scanner
        {
            get
            {
                if (_scanner == null)
                {
                    _scanner = CrossCSJPOSLib.CreateScanner();
                }

                return _scanner;
            }
        }



        public MainPage()
        {
            InitializeComponent();

            this.picker.SelectedIndex = 0;

            this.AddrEdit.Text = "";
            this.DisplayAddrEdit.Text = "";
            this.ScannerAddrEdit.Text = "";
            this.portEdit.Text = "9100";
            this.DisplayPortEdit.Text = "9200";
            this.ScannerPortEdit.Text = "9210";

            this.StatusText.Text = "Stopped";
            this.StatusFrame.BackgroundColor = Color.LightGray;

            InitializeActivityIndicator();
        }

        private void MessageBox(string message, string title)
        {
            var result = DisplayAlert(title, message, "OK");
        }

        private void OnSelectedChanged(object sender, EventArgs args)
        {
            var obj = (Picker)sender;
            if (2 == obj.SelectedIndex)
            {
                this.DisplayPortEdit.Text = "";
                this.ScannerPortEdit.Text = "";
            }
        }

        private int GetConnectType()
        {
            int result;
            switch(this.picker.SelectedIndex)
            {
                case 0:
                default:
                    result = ESCPOSConst.CMP_PORT_WiFi;
                    break;

                case 1:
                    result = ESCPOSConst.CMP_PORT_Bluetooth;
                    break;

                case 2:
                    result = ESCPOSConst.CMP_PORT_USB;
                    break;
            }
            return result;
        }



        private void OnSearchPrinter(object sender, EventArgs args)
        {
            SearchPrinter(GetConnectType());
        }

        private void OnPrinterConnect(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.AddrEdit.Text;
            port = this.portEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                PrinterConnect(GetConnectType(), address, portnum);
            }
            else
            {
                PrinterConnect(GetConnectType(), address);
            }
        }

        private void OnPrinterStatus(object sender, EventArgs args)
        {
            PrinterStatus();
        }

        private void OnSamplePrint(object sender, EventArgs args)
        {
            SamplePrint();
        }

        private void OnPrinterDisconnect(object sender, EventArgs args)
        {
            PrinterDisconnect();
        }

        private void OnDisplayConnect(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.DisplayAddrEdit.Text;
            port = this.DisplayPortEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                DisplayConnect(GetConnectType(), address, portnum);
            }
            else
            {
                DisplayConnect(GetConnectType(), address);
            }
        }

        private void OnSampleDisplay(object sender, EventArgs args)
        {
            SampleDisplay();
        }

        private void OnClearDisplay(object sender, EventArgs args)
        {
            ClearDisplay();
        }

        private void OnDisplayDisconnect(object sender, EventArgs args)
        {
            DisplayDisconnect();
        }

        private void OnScannerConnect(object sender, EventArgs args)
        {
            string address;
            string port;
            int portnum;
            address = this.ScannerAddrEdit.Text;
            port = this.ScannerPortEdit.Text;
            if (null == address) address = "";
            if (null == port) port = "";

            if (int.TryParse(port, out portnum))
            {
                ScannerConnect(GetConnectType(), address, portnum);
            }
            else
            {
                ScannerConnect(GetConnectType(), address);
            }
        }

        private void OnScannerDisconnect(object sender, EventArgs args)
        {
            ScannerDisconnect();
        }

        private async void SearchPrinter(int connectType)
        {
            String address = null;
            this.AddrEdit.Text = address;
            this.DisplayAddrEdit.Text = address;
            this.ScannerAddrEdit.Text = address;

            int searchTime;
            switch (connectType)
            {
                case ESCPOSConst.CMP_PORT_WiFi:
                    searchTime = 3;
                    break;

                case ESCPOSConst.CMP_PORT_Bluetooth:
                case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
                    searchTime = 10;
                    break;

                case ESCPOSConst.CMP_PORT_USB:
                    searchTime = 3;
                    break;

                default:
                    MessageBox("Search is not supported on that interface.", "Error");
                    searchTime = 10;
                    return;
            }
            int[] error = new int[1];
            CitizenPrinterInfo[] objs = null;

            IsVisivleActivityIndicator(true);

            await Task.Run(() => 
            {
                IESCPOSPrinter printer = Plugin.CSJPOSLib.CrossCSJPOSLib.CreateESCPOSPrinter();
                _ = printer.SetContext(this.platform.AndroidContext);
                objs = printer.SearchCitizenPrinter(connectType, searchTime, error);
            });

            IsVisivleActivityIndicator(false);

            if (error[0] == ESCPOSConst.CMP_SUCCESS)
            {
                System.Diagnostics.Debug.WriteLine("Debug ESCPOSPrinter#SearchCitizenPrinter()  Number of searches:" + objs.Length);


                List<String> list = new List<string>();
                foreach (CitizenPrinterInfo tmp in objs)
                {
                    switch (connectType)
                    {
                        case ESCPOSConst.CMP_PORT_WiFi:                            
                            list.Add(tmp.IpAddress);
                            break;

                        case ESCPOSConst.CMP_PORT_Bluetooth:
                        case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
                            list.Add(tmp.BdAddress);
                            break;

                        case ESCPOSConst.CMP_PORT_USB:
                            list.Add(tmp.UsbSerialNo);
                            break;
                    }
                }
                SelectItem(list.ToArray());
            }
            else
            {
                MessageBox("SearchCitizenPrinter()  error : " + error[0], "Error");
            }
        }



        int PrinterConnect(int connectType, string address, int port)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            if (this.Printer == null)
            {
                MessageBox("ESCPOSPrinter() : Instance is not created", "Error");
                return result;
            }

            this.Printer.SetContext(this.platform.AndroidContext);

            result = this.Printer.Connect(connectType, address, port);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }
            return result;
        }

        int PrinterConnect(int connectType, string address)
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            if (this.Printer == null)
            {
                MessageBox("ESCPOSPrinter() : Instance is not created", "Error");
                return result;
            }

            this.Printer.SetContext(this.platform.AndroidContext);

            result = this.Printer.Connect(connectType, address);

            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }
            return result;
        }

        int PrinterDisconnect()
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            if (this.Printer == null)
            {
                MessageBox("ESCPOSPrinter() : Instance is not created", "Error");
                return result;
            }

            result =this.Printer.Disconnect();
            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                MessageBox("disconnect() : Success", "Success");
            }
            else
            {
                MessageBox("disconnect()  error : " + result, "Error");
            }
            return result;
        }

        int PrinterStatus()
        {
            int result = ESCPOSConst.CMP_SUCCESS;
            string title = "";
            string message = "";

            if (this.Printer == null)
            {
                MessageBox("ESCPOSPrinter() : Instance is not created", "Error");
                return result;
            }

            result = this.Printer.PrinterCheck();
            if (result == ESCPOSConst.CMP_SUCCESS)
            {
                title = "Status";
                message = "printerCheck() : Success";

                int status = this.Printer.Status();
                if (ESCPOSConst.CMP_STS_NORMAL == status)
                {
                    title = "Status";
                    message = message + "\n\t * Normal ( No Error )";
                }
                else
                {
                    if ((ESCPOSConst.CMP_STS_COVER_OPEN & status) > 0)
                    {
                        title = "Status";
                        message = message + "\n\t * Cover Open";
                    }
                    if ((ESCPOSConst.CMP_STS_PAPER_EMPTY & status) > 0)
                    {
                        title = "Status";
                        message = message + "\n\t * Paper Empty";
                    }
                    if ((ESCPOSConst.CMP_STS_PRINTEROFF & status) > 0)
                    {
                        title = "Status";
                        message = message + "\n\t * Printer Offline";
                    }
                }
            }
            else
            {
                title = "Status";
                message = "printCheck()  error : " + result;
            }

            MessageBox(message, title);

            return result;
        }

        int SamplePrint()
        {
            int result = ESCPOSConst.CMP_SUCCESS;

            if (this.Printer == null)
            {
                MessageBox("ESCPOSPrinter() : Instance is not created", "Error");
                return result;
            }

            // printer check
            result = this.Printer.PrinterCheck();
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("PrinterCheck()  error : " + result, "Error");
                return result;
            }

            // get status
            int status = this.Printer.Status();
            if (ESCPOSConst.CMP_STS_NORMAL != status)
            {
                MessageBox("Status()  error : " + status, "Error");
                return result;
            }

            // character set
            result = this.Printer.SetEncoding("IBM437");        // OEM-US    Enable this line if your printer's MSW9-1 is "CP437".
        //  result = this.Printer.SetEncoding("UTF-8");         // Unicode   Enable this line if your printer supports "UTF-8" and prints characters in "UTF-8" code.
        //  result = this.Printer.SetEncoding("Shift_JIS");     // Japanese  { "Shift_JIS" R[hň󎚂ꍇ́A̍sLɂĂ.
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("SetEncoding()  error : " + result, "Error");
                return result;
            }

            // start transaction (batch)
            result = this.Printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("TransactionPrint()  error : " + result, "Error");
                return result;
            }

            // print text
            var appName = AppInfo.Name;
            string prndata = appName + "\n\n- Sample Print -\n1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n";
            result = this.Printer.PrintText(prndata, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("PrintText()  error : " + result, "Error");
                _ = this. Printer.ClearOutput();
                return result;
            }

            // print image
            Byte[] byteArray = ToBytes(ResourceLoader.GetObject("shoplogo.bmp"));
            result = this.Printer.PrintBitmap(byteArray, ESCPOSConst.CMP_BM_ASIS, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_BM_MODE_HT_DITHER);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("PrintBitmap()  error : " + result, "Error");
                _ = this.Printer.ClearOutput();
                return result;
            }

            // print image
            Object bitmap = platform.GetBitmap();
            result = this.Printer.PrintBitmap(bitmap, ESCPOSConst.CMP_BM_ASIS, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_BM_MODE_HT_DITHER);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("PrintBitmap()  error : " + result, "Error");
                _ = this.Printer.ClearOutput();
                return result;
            }

            // partical cut with pre-feed
            result = this.Printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("CutPaper()  error : " + result, "Error");
                _ = this.Printer.ClearOutput();
                return result;
            }

            // end transaction (batch)
            result = this.Printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);
            if (ESCPOSConst.CMP_SUCCESS != result)
            {
                MessageBox("TransactionPrint()  error : " + result, "Error");
                _ = this.Printer.ClearOutput();
                return result;
            }

            return result;
        }



        int DisplayConnect(int connectType, string address, int port)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            if (this.Display == null)
            {
                MessageBox("LineDisplay() : Instance is not created", "Error");
                return result;
            }

            result = this.Display.Connect(connectType, address, port);

            if (LineDisplayConst.CDP_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }

            return result;
        }

        int DisplayConnect(int connectType, string address)
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            if (this.Display == null)
            {
                MessageBox("LineDisplay() : Instance is not created", "Error");
                return result;
            }

            result = this.Display.Connect(connectType, address);

            if (LineDisplayConst.CDP_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }

            return result;
        }

        int DisplayDisconnect()
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            if (this.Display == null)
            {
                MessageBox("LineDisplay() : Instance is not created", "Error");
                return result;
            }

            result = this.Display.Disconnect();
            if (LineDisplayConst.CDP_SUCCESS == result)
            {
                MessageBox("Disconnect() : Success", "Success");
            }
            else
            {
                MessageBox("Disconnect()  error : " + result, "Error");
            }

            return result;
        }

        int SampleDisplay()
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            if (this.Display == null)
            {
                MessageBox("LineDisplay() : Instance is not created", "Error");
                return result;
            }

            result = this.Display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
            if (LineDisplayConst.CDP_SUCCESS != result)
            {
                MessageBox("clearDisplay()  error : " + result, "Error");
                return result;
            }

        //  _ = this.Display.SetEncoding("US-ASCII");      // US-ASCII  Enable this line if the display data is ASCII code only. 
        //  _ = this.Display.SetEncoding("Shift_JIS");     // Japanese  { "Shift_JIS" R[hŕ\ꍇ́A̍sLɂĂ.

            result = this. Display.DisplayText("Thank you so much!");
            if (LineDisplayConst.CDP_SUCCESS != result)
            {
                MessageBox("displayText()  error : " + result, "Error");
                return result;
            }

            result = this.Display.SetCursorPosition(13, 2);
            if (LineDisplayConst.CDP_SUCCESS != result)
            {
                MessageBox("setCursorPosition()  error : " + result, "Error");
                return result;
            }

            result = this.Display.DisplayText("$299.99-", true);
            if (LineDisplayConst.CDP_SUCCESS != result)
            {
                MessageBox("displayText()  error : " + result, "Error");
                return result;
            }

            return result;
        }

        int ClearDisplay()
        {
            int result = LineDisplayConst.CDP_SUCCESS;

            if (this.Display == null)
            {
                MessageBox("LineDisplay() : Instance is not created", "Error");
                return result;
            }

            result = this.Display.ClearDisplay(LineDisplayConst.CDP_AREA_ALL);
            if (LineDisplayConst.CDP_SUCCESS != result)
            {
                MessageBox("clearDisplay()  error : " + result, "Error");
                return result;
            }

            return result;
        }



        int ScannerConnect(int connectType, string address, int port)
        {
            int result = ScannerConst.CSC_SUCCESS;

            if (this.Scanner == null)
            {
                MessageBox("Scanner() : Instance is not created", "Error");
                return result;
            }

            this.Scanner.SetStatusUpdateEventCallback(StatusUpdateEventHandler);
            this.Scanner.SetDataEventCallback(DataEventHandler);

            result = this.Scanner.Connect(connectType, address, port);

            if (ScannerConst.CSC_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }

            return result;
        }

        int ScannerConnect(int connectType, string address)
        {
            int result = ScannerConst.CSC_SUCCESS;

            if (this.Scanner == null)
            {
                MessageBox("Scanner() : Instance is not created", "Error");
                return result;
            }

            this.Scanner.SetStatusUpdateEventCallback(StatusUpdateEventHandler);
            this.Scanner.SetDataEventCallback(DataEventHandler);

            result = this.Scanner.Connect(connectType, address);

            if (ScannerConst.CSC_SUCCESS == result)
            {
                MessageBox("Connect() : Success", "Success");
            }
            else
            {
                MessageBox("Connect()  error : " + result, "Error");
            }

            return result;
        }

        int ScannerDisconnect()
        {
            int result = ScannerConst.CSC_SUCCESS;

            if (this.Scanner == null)
            {
                MessageBox("Scanner() : Instance is not created", "Error");
                return result;
            }

            this.Scanner.SetStatusUpdateEventCallback(null);
            this.Scanner.SetDataEventCallback(null);

            this.StatusText.Text = "Stopped";
            this.StatusFrame.BackgroundColor = Color.LightGray;

            result = this.Scanner.Disconnect();
            if (ScannerConst.CSC_SUCCESS == result)
            {
                MessageBox("Disconnect() : Success", "Success");
            }
            else
            {
                MessageBox("Disconnect()  error : " + result, "Error");
            }
            return result;
        }

        public void DataEventHandler(byte[] data)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                System.Diagnostics.Debug.WriteLine("OnDataEvent");

                string tmp = System.Text.Encoding.GetEncoding("us-ascii").GetString(data);      // US-ASCII  Enable this line if the scan data is ASCII code only.
            //  string tmp = System.Text.Encoding.GetEncoding("shift_jis").GetString(data);     // Japanese  XL f[^ "Shift_JIS" R[h̓{ꂪ܂܂ꍇ́A̍sLɂĂ.

                MessageBox(tmp, "Scan Call back:");
            });
        }

        public void StatusUpdateEventHandler(int status)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                if (ScannerConst.CSC_SUE_POWER_ONLINE == status)
                {
                    this.StatusText.Text = "Online";
                    this.StatusFrame.BackgroundColor = Color.FromRgb(0, 255, 0);
                }
                else
                {
                    this.StatusText.Text = "Off-line";
                    this.StatusFrame.BackgroundColor = Color.Red;
                }
            });
        }



        private async void SelectItem(String[] items)
        {
            String title = "Found printer";
            String cancel = "Cancel";
            String destruction = null;  // "Destruction";
            var result = await this.DisplayActionSheet(title, cancel, destruction, items);
            if ((result != null) && !(result.Equals(cancel)) && !(result.Equals(destruction)))
            {
                this.AddrEdit.Text = result;
                this.DisplayAddrEdit.Text = result;
                this.ScannerAddrEdit.Text = result;
            }
        }



        private void IsVisivleActivityIndicator(Boolean isVisible)
        {
            if (this.frameActivityIndicatorLayer == null)
            {
                return;
            }
            this.frameActivityIndicatorLayer.IsVisible = isVisible;
            this.IsEnabled = !isVisible;
        }
        private void InitializeActivityIndicator()
        {
            SizeChanged += AbsolutePageCS_SizeChanged;
        }
        private void AbsolutePageCS_SizeChanged(object sender, EventArgs e)
        {
            SizeChangeActivityIndicator();
        }
        private void SizeChangeActivityIndicator()
        { 
            if (this.frameActivityIndicatorLayer == null)
            {
                return;
            }
            double width;
            switch (Device.RuntimePlatform)
            {
                case Device.iOS:
                case Device.Android:
                    width = AbsoluteLayout.AutoSize;
                    break;

                default:
                    width = this.Width;
                    break;
            }
            AbsoluteLayout.SetLayoutFlags(this.frameActivityIndicatorLayer, AbsoluteLayoutFlags.PositionProportional);
            AbsoluteLayout.SetLayoutBounds(this.frameActivityIndicatorLayer, new Rectangle(0.5d, 0.5d, width, AbsoluteLayout.AutoSize));
        }



        private byte[] ToBytes(System.IO.Stream stream)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                try
                {
                    stream.CopyTo(ms);
                    stream.Close();
                    byte[] byteArray = ms.ToArray();
                    return byteArray;
                }
                catch
                {
                    return null;
                }
            }
        }


        public static class ResourceLoader
        {
            static internal string[] Names { get; set; }
            static internal Assembly Assembly { get; set; }

            public static System.IO.Stream GetObject(string resourceName)
            {
                if (ResourceLoader.Assembly == null)
                {
                    ResourceLoader.Assembly = typeof(ResourceLoader).GetTypeInfo().Assembly;
                    ResourceLoader.Names = ResourceLoader.Assembly.GetManifestResourceNames();
                    foreach(String name in ResourceLoader.Names) {
                        System.Diagnostics.Debug.WriteLine("found resource: " + name);
                    }
                }
                try
                {
                    string path = ResourceLoader.Names.First(x => x.EndsWith(resourceName, StringComparison.CurrentCultureIgnoreCase));
                    return ResourceLoader.Assembly.GetManifestResourceStream(path);
                }
                catch
                {
                    return null;
                }
            }
        }
    }
}
